// Windows Scripting Host wrapper

if (!String.prototype.format) {
	String.prototype.format = function () {
		var args = arguments;
		return this.replace(/{(\d+)}/g, function (match, number) {
			return args[number] != undefined ? args[number] : match;
		});
	};
} //if !String.prototype.format

var CommandLineParser = function() {
	var CommandLineOption = function(/*string*/ argument) {
		this.key = null;
		this.value = argument;
		this.isFile = true;
		this.isOn = false;
		this.isOff = false;
		this.isDefault = false;
		var matches = argument.match(/^[\-\/](.+?)([:\+\-])(.*)$/);
		this.isFile = matches == null;
		if (!this.isFile) {
			this.key = matches[1];
			var indicator = matches[2];
			this.value = matches[3];
			this.isOn = indicator == '+';
			this.isOff = indicator == '-';
			this.isDefault = indicator == "";
		} // if !file
		if (this.isFile) this.type = "file";
		else if (this.isOn) this.type = "+";
		else if (this.isOff) this.type = "-";
		else if (this.isDefault) this.type = "default";
		else this.type = "value";
	}; //CommandLineOption
	this.files = [];
	this.optionSet = [];
	this.redundantOptionSet = [];
	var arguments = WScript.Arguments;
	for (var index = 0; index < arguments.length; ++index) {
		var option = new CommandLineOption(arguments(index));
		if (!option.isFile) {
			var isRedundant = this.optionSet.hasOwnProperty(option.key);
			if (!isRedundant) {
				this.optionSet.push(option);
				this.optionSet[option.key] = option;
			} else		
				this.redundantOptionSet.push(option);
		} else
			this.files.push(option.value);
	} //loop
	this.findOption = function(key) {
		return this.optionSet[key];
	}; //findOption
	this.getValue = function(key) {
		var option = this.optionSet[key];
		if (!option) return undefined;
		if (option.isFile) return null;
		if (option.isOn) return null;
		if (option.isOff) return null;
		if (option.isDefault) return null;
		return option.value;
	}; //getValue
	this.getType = function(key) {
		var option = this.optionSet[key];
		if (!option) return undefined;
		return option.type;
	}; //getType
}; //CommandLineParser

var FileSystem = {

	implementation: new ActiveXObject("Scripting.FileSystemObject"),
	
	defaultOutputFileOption: 'o',
	specialFolderIndex: 2,
	fileForReadingIndex: 1,
	fileForWritingIndex: 2,
	fileForAppendingIndex: 8,
	temporaryFileName: null,

	getTemporaryFile: function() {
		this.temporaryFileName = this.implementation.BuildPath(this.implementation.GetSpecialFolder(this.specialFolderIndex), this.implementation.GetTempName());
		return this.temporaryFileName;
	}, //getTemporaryFile
	deleteTemporaryFile: function() {
	  	if (this.implementation.FileExists(this.temporaryFileName))
    			this.implementation.DeleteFile(this.temporaryFileName);
		this.temporaryFileName = null
	}, //deleteTemporaryFile
	readAllText: function(fileName) {
		var ts = this.implementation.OpenTextFile(fileName, this.fileForReadingIndex, false);
		result = ts.ReadAll();
		ts.Close();
		return result;
	}, //readAllText
	readAllTextFromTemporaryFile: function() {
	  	if (this.implementation.FileExists(this.temporaryFileName))
			return this.readFile(this.temporaryFileName);
	}, //readAllTextFromTemporaryFile
	writeAllText: function(fname, content) {
		var ts = this.implementation.CreateTextFile(fname, true);
		ts.Write(content);
		ts.Close();
	}, //writeAllText
	expandFileName: function(simpleName) {
		return this.implementation.GetAbsolutePathName(simpleName);
	}, //expandFileName

	requireInputFilesGetOneOutputFile: function(/*int*/ inputCount, /*string*/ ext) {
		var parser = new CommandLineParser();
		if (parser.files.length != inputCount) {
			var multiplicity = "s";
			if (inputCount == 1) multiplicity = "";
			return "Required {0} input file{1}, {2} found.\nFor an output file, use the option -{3}:<file_name>.".format(
				inputCount, multiplicity, parser.files.length, this.defaultOutputFileOption);
		} //if
		var notFoundFiles = [];
		for (var index in parser.files) {
			if (!this.implementation.FileExists(parser.files[index]))
				notFoundFiles.push(parser.files[index]);
		} //loop
		if (notFoundFiles.length > 0) {
			var message = "";
			var multiplicity = "s";
			if (notFoundFiles.length == 1) multiplicity = "";
			for (var index in notFoundFiles)
				message += "\n" + notFoundFiles[index];
			return "File{0} not found:\n{1}".format(multiplicity, message);
		} //if not found 
		var outputFile = parser.getValue(this.defaultOutputFileOption);
		var outputFileAutogenerated = false;
		if (!outputFile && ext && parser.files.length > 0) {
			outputFile = this.implementation.GetBaseName(parser.files[0]) + ext;
			outputFileAutogenerated = true; 
		} // if !outputFile
		return { files: parser.files, outputFile: outputFile, outputFileAutogenerated: outputFileAutogenerated }; 
	} //requireInputFilesGetOneOutputFile
	
}; //FileSystem

var MessageBoxOptions = {
	buttonsOK:0,
	buttonsOKCancel:1,
	buttonsAbortRetryIgnore:2, 
	buttonsYesNoCancel:3,
	buttonsYesNo:4,
	buttonsRetryCancel:5,
	iconCritical:16,
        iconQuestion:32,
        iconExclamation:48,
	iconInformation:64,
	defaultButton1:0,
	defaultButton2:256, 
	defaultButton3:512
}; //MessageBoxOptions

var Shell = {

	implementation: new ActiveXObject("WScript.Shell"),	

	run: function(/*String*/ cmd, /*Boolean*/ wait, /*String*/ title, /*Boolean*/ noClose) {
		if (title || noClose) {
			var comspecOption = noClose ? 'K' : 'C';
			if (!title) title = "";
			cmd = "%comspec% /{0} title {1} & {2}".format(comspecOption, title, cmd);
		} //if use %comspec%
		this.implementation.run(cmd, 1, wait);
	}, //run

	messageBox: function(message, title, /*MessageBoxOptions*/ options) {
		this.implementation.Popup(message, 0, title, options);
	}, //messageBox

	errorBox: function(message, title) {
		this.implementation.Popup(message, 0, title, MessageBoxOptions.iconCritical | MessageBoxOptions.buttonsOK);
	} //errorBox

}; //Shell
